//+------------------------------------------------------------------+
//|                     Kaufman Adaptive Moving Average with signals |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Red
#property indicator_color3  Gold
#property indicator_width1  3
#property indicator_width2  3
#property indicator_width3  3
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern int    AMAPeriod       = 26;
extern int    AMAPrice        = PRICE_CLOSE;
extern int    Nfast           = 5;
extern int    Nslow           = 20;
extern double GCoeff          = 2.0;
extern double dK              = 2.0; 
extern bool   ShowNeutral     = true;
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";

//
//
//
//
//

double kAMAbuffer[];
double kAMAup[];
double kAMAdn[];
double kAMAnu[];
double diff[];
double trend[];

//
//
//
//
//

double fastend;
double slowend;
string priceName;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,kAMAup);    SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,kAMAdn);    SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,kAMAnu);    SetIndexStyle(2,DRAW_HISTOGRAM);   
   SetIndexBuffer(3,kAMAbuffer);
   SetIndexBuffer(4,diff);
   SetIndexBuffer(5,trend);
   
   
   //
   //
   //
   //
   //
   
      fastend   = (2.0 /(Nfast + 1));
      slowend   = (2.0 /(Nslow + 1));
      switch(AMAPrice)
      {
         case PRICE_CLOSE:    priceName = "Close";    break;  // 0
         case PRICE_OPEN:     priceName = "Open";     break;  // 1
         case PRICE_HIGH:     priceName = "High";     break;  // 2
         case PRICE_LOW:      priceName = "Low";      break;  // 3
         case PRICE_MEDIAN:   priceName = "Median";   break;  // 4
         case PRICE_TYPICAL:  priceName = "Typical";  break;  // 5
         case PRICE_WEIGHTED: priceName = "Weighted"; break;  // 6
      }      
   
   //
   //
   //
   //
   //
   
   IndicatorShortName("Kaufman AMA signals ("+AMAPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);

           //
           //
           //
           //
           //
                      
           if (Digits==3 || Digits==5)
              double _point=Point*10;
           else      _point=Point;         


   //
   //
   //
   //
   //
   
   for(int i=limit; i>=0; i--)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,AMAPrice,i);

         if (i > Bars-AMAPeriod) { kAMAbuffer[i] = price; continue; }
   
         //
         //
         //
         //
         //
   
         double efratio = 1.00;
         double noise   = 0.00;
         double smooth;
         double signal;
             signal  = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,AMAPrice,i+AMAPeriod));
             diff[i] = MathAbs(price-iMA(NULL,0,1,0,MODE_SMA,AMAPrice,i+1));
                for (int k=0;k<AMAPeriod;k++)
                        noise += diff[i+k];

          //
          //
          //
          //
          //

          if (noise != 0) efratio = signal/noise;
                          smooth        = MathPow(efratio*(fastend-slowend)+slowend,GCoeff);
                          kAMAbuffer[i] = kAMAbuffer[i+1] + smooth*(price-kAMAbuffer[i+1]);

          //
          //
          //
          //
          //
         
         if (ShowNeutral)
              trend[i] = 0;
         else trend[i] = trend[i+1];                           
         kAMAup[i] = EMPTY_VALUE;
         kAMAdn[i] = EMPTY_VALUE;
         kAMAnu[i] = EMPTY_VALUE;
            double ddK=kAMAbuffer[i] - kAMAbuffer[i+1];
               if ((MathAbs(ddK)) > (dK*_point) && (ddK > 0)) trend[i] = 1;
               if ((MathAbs(ddK)) > (dK*_point) && (ddK < 0)) trend[i] =-1;
               if ((MathAbs(ddK))== (dK*_point) && (ddK== 0)) trend[i] = 0;
               if (trend[i] == 1) kAMAup[i] = 1;
               if (trend[i] ==-1) kAMAdn[i] = 1;
               if (trend[i] == 0) kAMAnu[i] = 1;
                             
   }

   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("Buy");
      else  doAlert("Sell");       
   }
   return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," AMA ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," AMA "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}